      subroutine ardrive(N, Nz, AA, JA, IA,
     1            nev, ncv, lworkl, resid, v, workd, workl, info, ido,
     2            selection, d, ierr, 
     3            llen, larr, lcol, lrow,
     4            ulen, uarr, ucol, urow, maxit)
c* Driver for ARPACK: all calculations are done in FORTRAN
c* memory allocations done in C
c* solves the Generalized eigenvalue problem Ax=lambda. Bx
c* B - symmetric positive definite, A - symmetric
c* Arguments
c* N - matrix size
c* Nz- non zero elements in A,
c* AA, JA, IA - A in CSR format
c* nev, ncv, lworkl, resid, v, workd, workl, info, ido, selection, d, ierr
c* - all the above are usual parameters in ARPACK
c* B = L U using cholesky decomposition
c* llen - non zero elements of L
c* larr, lcol, lrow - L in CSR format
c* ulen - non zero elements of U
c* uarr, ucol, urow - U in CSR format
c* maxit - max outer iterations of ARPACK

c* ================================================================
c*  pdnmesh - a 2D finite element solver
c*  Copyright (C) 2001-2005 Sarod Yatawatta <sarod@users.sf.net>  
c* $Id: ardrive.f,v 1.3 2005/04/19 00:32:25 sarod Exp $
c* ================================================================

c* Arguments
      integer N, Nz, JA(Nz), IA(N), 
     1        nev, ncv, lworkl, info, ido, 
     2        llen, lcol(llen), lrow(N), ulen, ucol(ulen), urow(N), 
     3        maxit
      double precision AA(Nz), x(N), y(N), resid(N), v(N,ncv),
     1  workd(3*N), workl(lworkl), selection(ncv), d(ncv,2),
     2  larr(llen), uarr(ulen)
c* Local variables
      integer ldv, ipntr(11), iparam(11) 
      double precision tol, sigma
      character bmat*1, which*2
      logical rvec

c* Initialize Locals
      bmat ='I'
      which='SM'
      tol=0.0D+0
      ldv=N
c* set globals
      info=0
      ido=0
c* Set Algorithm Mode
      iparam(1)=1
      iparam(3)=maxit
      iparam(7)=1
c* Main loop
   10 continue
      call dsaupd(ido, bmat, N, which, nev, tol, resid,
     1       ncv, v, ldv, iparam, ipntr, workd, workl,
     2       lworkl, info)

      if (ido .eq. -1 .or. ido .eq. 1) then
c*        call matproduct(N,Nz,AA,JA,IA,workd(ipntr(1)),workd(ipntr(2)))
        call shiftinvsolve(N,Nz,AA,JA,IA,
     1     llen, larr, lcol, lrow,
     2     ulen, uarr, ucol, urow,
     3     workd(ipntr(1)),workd(ipntr(2)))
        go to 10
      end if

c* If no errors, calculate eigenvectors
      if (info .eq. 0) then
        rvec =.true.
        call dseupd ( rvec, 'All', selection, d, v, ldv, sigma,
     &         bmat, n, which, nev, tol, resid, ncv, v, ldv,
     &         iparam, ipntr, workd, workl, lworkl, ierr )

      end if
c* end of program
      return
      end


      subroutine shiftinvsolve(N, Na, AA, JA, IA,
     1                        Nl, LL, JL, IL,
     2                        Nu, UU, JU, IU, x, y)

c* Arguments
       integer N, Na, JA(Na), IA(N), Nl, JL(Nl), IL(N),
     1         Nu, JU(Nu), IU(N)
       double precision AA(Na), LL(Nl), UU(Nu), x(N), y(N)

c* Local variables
       double precision z(N), v(N)
c* Step 1: solve L^T z = x for z using backsubstitution

       call backsubstitute(N,Nu,UU,JU,IU,z,x)
c* Step 2: multiply z by A, v=Az
       call matproduct(N,Na,AA,JA,IA,z,v)
c* Step 3: solve L y = v for y by forward elimination
       call forwardelim(N,Nl,LL,JL,IL,y,v)

       return
       end


      subroutine matproduct(N, Nz, AA, JA, IA, x, y)
c*  Calculate matrix product Ax=>y, A is given in CSR format
c*  Array indices are given in Fortran format, so runs from 1 to N, unlike
c*  C format.

c* Arguments
      integer N, Nz, JA(Nz), IA(N)
      double precision AA(Nz), x(N), y(N)
c* Local variables
      integer i, k1, k2, k
      do 10 i=1,N-1
        k1 = IA(i)
        k2 = IA(i+1)-1
        y(i)=0.0
        do 20 k=k1,k2
         y(i) = y(i) + AA(k)*x(JA(k))
   20   continue
   10 continue   
c* last row
      i  = N
      k1 = IA(i)
      k2 = Nz 
      y(i)=0.0
      do 30 k=k1,k2
        y(i) = y(i) + AA(k)*x(JA(k))
   30  continue
c* end subroutine
      return
      end


      subroutine forwardelim(N,Nz,AA,JA,IA,x,y)
c*  Solve Ax = y by forward elimination, A is 
c*  Lower triangular.
c*  Arguments
      integer N, Nz, JA(Nz), IA(N)
      double precision AA(Nz), x(N), y(N)
c*  Local variables
      integer i,k,k1,k2;
      double precision temp,ll0;

      do 10 i=1,N
         k1=IA(i)
         if (i .lt. N) then
          k2=IA(i+1)-1
         else
          k2=Nz
         endif
         ll0=1.0
         temp=0.0
         do 20 k=k1,k2
           if (JA(k) .ne. i) then
             temp =temp + AA(k)*x(JA(k))
           else
             ll0 = AA(k)
           endif
   20   continue
         x(i) = (y(i)-temp)/ll0
   10 continue

      return
      end



      subroutine backsubstitute(N,Nz,AA,JA,IA,x,y)
c* Solve Ax = y by backward substitution. A is 
c* Upper triangular.
c*  Arguments
      integer N, Nz, JA(Nz), IA(N)
      double precision AA(Nz), x(N), y(N)
c*  Local variables
      integer i,k,k1,k2;
      double precision temp,ll0;

      do 10 i=N,1,-1
         k1=IA(i)
         if (i .lt. N) then
          k2=IA(i+1)-1
         else
          k2=Nz
         endif
         ll0=1.0
         temp=0.0
         do 20 k=k1,k2
           if (JA(k) .ne. i) then
             temp =temp + AA(k)*x(JA(k))
           else
             ll0 = AA(k)
           endif
   20   continue
         x(i) = (y(i)-temp)/ll0
   10 continue

      return
      end
